# -*- coding: utf-8 -*-

import numpy as np

class Dos:
    def __init__(self):
        self.n_edos = 0
        self.e_fermi = 0.0
    
    def read_DOSCAR(self, file_name = "DOSCAR"):
        with open(file_name) as f:
            list_energy = []
            list_dos = []
            list_tdos = []
            for i in range(5):
                f.readline()
            line = f.readline().strip().split()
            self.n_edos = int(line[2])
            self.e_fermi = float(line[3])
            for i in range(self.n_edos):
                line = f.readline()
                list_energy.append(float(line.strip().split()[0]))
                list_dos.append(float(line.strip().split()[1]))
                list_tdos.append(float(line.strip().split()[2]))
            self._energy = np.array(list_energy) - self.e_fermi
            self._dos = np.array(list_dos)
            self._tdos = np.array(list_tdos)
            
    def get_dos(self):
        return np.stack((self._energy, self._dos), axis = 1)
    
    def get_tdos(self):
        return np.stack((self._energy, self._tdos), axis = 1)



# -*- coding: utf-8 -*-

import numpy as np
from functools import reduce
import copy
from atom import Atom

class Structure:
    def __init__(self, a = 1.0, b = 1.0, c = 1.0, alpha = np.pi/2, beta = np.pi/2, gamma = np.pi/2):
        self._lattice = np.array([[a, 0.0, 0.0], [0.0, b, 0.0], [0.0, 0.0, c]])
        self._atom = []
        self._species = []
        self._coords_direct = []
        self._coords_cartesian = []
        self._forces = []
        self._energy = 0.0
        self._isFix = []
    
    def __str__(self):
        str_summary = 'Structure Summary\n'
        str_summary += 'Lattice\n'
        abc = self.get_abc()
        str_summary += '  abc:\t'
        str_summary += f'{abc[0]:.10f}\t{abc[1]:.10f}\t{abc[2]:.10f}\n'
        angles = self.get_angles()
        str_summary += '  ang:\t'
        str_summary += f'{angles[0]:.10f}\t{angles[1]:.10f}\t{angles[2]:.10f}\n'
        str_summary += '  vec_a:\t'
        str_summary += f'{self._lattice[0][0]:.10f}\t{self._lattice[0][1]:.10f}\t{self._lattice[0][2]:.10f}\n'
        str_summary += '  vec_b:\t'
        str_summary += f'{self._lattice[1][0]:.10f}\t{self._lattice[1][1]:.10f}\t{self._lattice[1][2]:.10f}\n'
        str_summary += '  vec_c:\t'
        str_summary += f'{self._lattice[2][0]:.10f}\t{self._lattice[2][1]:.10f}\t{self._lattice[2][2]:.10f}\n'
        str_summary += f'Atom  {len(self._species)} atom(s) in total\n'
        for i in range(len(self._species)):
            str_summary += f'  {self._species[i]}\t'
            str_summary += f'{self._coords_direct[i][0]:.10f}\t{self._coords_direct[i][1]:.10f}\t{self._coords_direct[i][2]:.10f}\n'
        return str_summary
    
    def __repr__(self):
        return self.__str__()
    
    def get_atom(self, i):
        if(i < len(self._atom)):
            return self._atom[i]
    
    def get_abc(self):
        return np.sqrt((self._lattice * self._lattice).sum(axis = 1))
    
    def get_angles(self):
        abc = np.sqrt((self._lattice * self._lattice).sum(axis = 1))
        alpha = np.arccos(self._lattice[1].dot(self._lattice[2])/(abc[1] * abc[2]))
        beta = np.arccos(self._lattice[0].dot(self._lattice[2])/(abc[0] * abc[2]))
        gamma = np.arccos(self._lattice[0].dot(self._lattice[1])/(abc[0] * abc[1]))
        return np.array([alpha, beta, gamma]) * 180 / np.pi
    
    def read_POSCAR(self, file_name = 'POSCAR'):
        temp_lattice = []
        temp_coords_direct = []
        temp_coords_cartesian = []
        isFix = False
        
        with open(file_name) as f:
            for i in range(2):
                f.readline()
            for i in range(3):
                data = f.readline().strip().split()
                data = list(map(float, data))
                temp_lattice.append(data)
            self._lattice = np.array(temp_lattice)
            list_species = f.readline().strip().split()
            num_atom = f.readline().strip().split()
            num_atom = list(map(int, num_atom))
            for index, value in enumerate(list_species):
                self._species += [value] * num_atom[index]
                self._atom += [Atom(value)] * num_atom[index]  
            isDirect = f.readline().strip()
            if(isDirect[0] == 'S'):
                isFix = True
                isDirect = f.readline().strip()
            for i in range(int(np.sum(num_atom))):
                data = f.readline().split()
                if(isFix):
                    if(isDirect[0] in ['C', 'c', 'K', 'k']):
                        temp_coords_cartesian.append(data[:3])
                    else:
                        temp_coords_direct.append(data[:3])
                    self._isFix.append(data[3:])
                else:
                    if(isDirect[0] in ['C', 'c', 'K', 'k']):
                        temp_coords_cartesian.append(data)
                    else:
                        temp_coords_direct.append(data)
                    self._isFix.append(['T','T','T'])
            if(isDirect[0] in ['C', 'c', 'K', 'k']):
                self._coords_cartesian = np.array(temp_coords_cartesian).astype('float64')
            else:
                self._coords_direct = np.array(temp_coords_direct).astype('float64')
        self.fillna()
    
    def fillna(self):
        if(len(self._coords_cartesian) > len(self._coords_direct)):
            self._coords_direct = self._coords_cartesian @ np.linalg.inv(self._lattice)
        elif(len(self._coords_direct) > len(self._coords_cartesian)):
            self._coords_cartesian = self._coords_direct @ self._lattice
        if(len(self._forces) != len(self._species)):
            self._forces = np.zeros(self._coords_direct.shape)
        if(len(self._isFix) != len(self._species)):
            self._isFix = np.array([['T','T','T'] for i in range(len(self._coords_direct))])
    
    def update_cartesian(self):
        self._coords_cartesian = self._coords_direct @ self._lattice
    
    def update_direct(self):
        self._coords_direct = self._coords_cartesian @ np.linalg.inv(self._lattice)
    
    def cal_dist(self, i = 0, j = 0):
        num_atom = len(self._coords_cartesian)
        if(i < 0 or j < 0 or i >= num_atom or j >= num_atom):
            print('Error: atom number out of index...')
            return
        else:
            x2 = (self._coords_cartesian[i][0] - self._coords_cartesian[j][0]) * (self._coords_cartesian[i][0] - self._coords_cartesian[j][0])
            y2 = (self._coords_cartesian[i][1] - self._coords_cartesian[j][1]) * (self._coords_cartesian[i][1] - self._coords_cartesian[j][1])
            z2 = (self._coords_cartesian[i][2] - self._coords_cartesian[j][2]) * (self._coords_cartesian[i][2] - self._coords_cartesian[j][2])
            return np.sqrt(x2 + y2 + z2)
    
    def cal_volume(self):
        vol = np.sum(np.cross(self._lattice[0], self._lattice[1]) * self._lattice[2])
        return abs(vol)
    
    def re_orientation(self):
        pass
    
    def incell(self):
        while((self._coords_direct > 1).any()):
            self._coords_direct[np.where(self._coords_direct > 1.0)] -= 1
        while((self._coords_direct < 0).any()):
            self._coords_direct[np.where(self._coords_direct < 1.0)] += 1
        self.update_cartesian()
    
    def fix(self, threshold = 0.0, direction = 'z'):
        for i in range(len(self._coords_direct)):
            if(self._coords_direct[i][2] < threshold):
                self._isFix[i] = ['F','F','F']
            else:
                self._isFix[i] = ['T','T','T']
    
    def supercell(self, a = 1, b = 1, c = 1):  # do not finish
        if(a <= 0 or b <= 0 or c <= 0):
            print("ERROR: positive multiples are required")
            return
        a, b, c = int(a), int(b), int(c)
        self._lattice[0] *= a
        self._lattice[1] *= b
        self._lattice[2] *= c
        self._species *= a*b*c
        self._coords_direct[:, 0] /= a
        self._coords_direct[:, 1] /= b
        self._coords_direct[:, 2] /= c
        coords_copy = copy.deepcopy(self._coords_direct)
        for i in range(a):
            for j in range(b):
                for k in range(c):
                    if(i or j or k):
                        self._coords_direct = np.concatenate((self._coords_direct, coords_copy + (i/a, j/b, k/c)))
        del coords_copy
        self.update_cartesian()
        force_copy = copy.deepcopy(self._forces)
        for i in range(a*b*c):
            self._forces = np.concatenate((self._forces, force_copy))
        del force_copy
        self._isFix *= a*b*c
    
    def to_POSCAR(self, file_name = 'POSCAR', mode = 'Direct'):
        isFix = False
        
        with open(file_name, 'w') as f:
            f.write('POSCAR_created_by_datana\n')
            f.write('1.0000000000\n')
            f.write(f'\t{self._lattice[0][0]:.10f}\t{self._lattice[0][1]:.10f}\t{self._lattice[0][2]:.10f}\n')
            f.write(f'\t{self._lattice[1][0]:.10f}\t{self._lattice[1][1]:.10f}\t{self._lattice[1][2]:.10f}\n')
            f.write(f'\t{self._lattice[2][0]:.10f}\t{self._lattice[2][1]:.10f}\t{self._lattice[2][2]:.10f}\n')
            reduce_species = reduce(lambda x, y: x if y in x else x + [y], [[], ] + self._species)
            for i in range(len(reduce_species)):
                f.write(f'  {reduce_species[i]}')
            f.write('\n')
            for i in range(len(reduce_species)):
                f.write(f'  {self._species.count(reduce_species[i])}')
            f.write('\n')
            if(np.sum(np.array(self._isFix) == 'F')):
                f.write('Selective dynamics\n')
                isFix = True
            f.write(f'{mode}\n')
            if(isFix):
                if(mode == 'Cartesian'):
                    for i in range(len(self._coords_cartesian)):
                        f.write(f'  {self._coords_cartesian[i][0]:.10f}  {self._coords_cartesian[i][1]:.10f}  {self._coords_cartesian[i][2]:.10f}  {self._isFix[i][0]}  {self._isFix[i][1]}  {self._isFix[i][2]}\n')
                else:
                    for i in range(len(self._coords_direct)):
                        f.write(f'  {self._coords_direct[i][0]:.10f}  {self._coords_direct[i][1]:.10f}  {self._coords_direct[i][2]:.10f}  {self._isFix[i][0]}  {self._isFix[i][1]}  {self._isFix[i][2]}\n')
            else:
                if(mode == 'Cartesian'):
                    for i in range(len(self._coords_cartesian)):
                        f.write(f'  {self._coords_cartesian[i][0]:.10f}  {self._coords_cartesian[i][1]:.10f}  {self._coords_cartesian[i][2]:.10f}\n')
                else:
                    for i in range(len(self._coords_direct)):
                        f.write(f'  {self._coords_direct[i][0]:.10f}  {self._coords_direct[i][1]:.10f}  {self._coords_direct[i][2]:.10f}\n')

class Structures:
    def __init__(self):
        self._structures = []

    def __str__(self):
        str_summary = 'Structures Summary\n'
        str_summary += f'Frames:\t{len(self._structures)}'
        return str_summary
        
    def __repr__(self):
        return self.__str__()
    
    def add(self, struct):
        if(type(struct) == Structure):
            self._structures.append(struct)
        else:
            print('type input error...')
    
    def pop(self):
        try:
            self._structures.pop()
        except:
            print('pop error...')
    
    def read_OUTCAR(self, file_name = "OUTCAR", verbose = 1):
        with open(file_name) as f:
            s_single = Structure()
            list_species = []
            num_atom = [0]
            is_read_lattice_vector = False  # do not read in first time
            
            if(verbose):
                print('Reading the OUTCAR file...')
            for line in f:
                if("VRHFIN" in line):
                    list_species.append(line.split()[1].replace("=", "").replace(":", ""))
                elif("ions per type" in line):
                    num_atom = [int(i) for i in line.split()[4:]]
                if(sum(num_atom) > 0 and not s_single._species):
                    for index, value in enumerate(list_species):
                        s_single._species += [value] * num_atom[index]
                        s_single._atom += [Atom(value)] * num_atom[index]
                
                if("direct lattice vectors" in line):
                    if(not is_read_lattice_vector):
                        is_read_lattice_vector = True
                        continue
                    list_lattice = []
                    for i in range(3):
                        line = f.readline()
                        list_line = line.split()
                        list_lattice.append([float(list_line[0]), float(list_line[1]), float(list_line[2])])
                    s_single._lattice = np.array(list_lattice)
                elif("POSITION" in line and "TOTAL-FORCE" in line):
                    positions_single = []
                    forces_single = []
                    line = f.readline()
                    line = f.readline()
                    for i in range(sum(num_atom)):
                        list_line = line.strip('\n').split()
                        positions_single.append([float(list_line[0]), float(list_line[1]), float(list_line[2])])
                        forces_single.append([float(list_line[3]), float(list_line[4]), float(list_line[5])])
                        line = f.readline()
                    s_single._coords_cartesian = np.array(positions_single)
                    s_single._forces = np.array(forces_single)
                    for i in range(12):
                        line = f.readline()
                    s_single._energy = float(line.strip('\n').split()[-1])
                    s_single.fillna()
                    s_single.update_direct()
                    s_single_copy = copy.deepcopy(s_single)
                    self._structures.append(s_single_copy)
                    if(verbose):
                        print(f'\r{len(self._structures)}', end = '')
        if(verbose):
            print()
    
    def read_XDATCAR(self, file_name = "XDATCAR"):
        with open(file_name) as f:
            s_single = Structure()
            temp_lattice = []
            for i in range(2):
                f.readline()
            for i in range(3):
                data = f.readline().strip().split()
                data = list(map(float, data))
                temp_lattice.append(data)
            s_single._lattice = np.array(temp_lattice)
            list_species = f.readline().strip().split()
            num_atom = f.readline().strip().split()
            num_atom = list(map(int, num_atom))
            for index, value in enumerate(list_species):
                s_single._species += [value] * num_atom[index]
                s_single._atom += [Atom(value)] * num_atom[index]
            for line in f:
                temp_coords_direct = []
                for i in range(int(np.sum(num_atom))):
                    data = f.readline().split()
                    temp_coords_direct.append(data)
                s_single._coords_direct = np.array(temp_coords_direct).astype('float64')
                s_single.update_cartesian()
                s_single_copy = copy.deepcopy(s_single)
                self._structures.append(s_single_copy)
    
    def read_xyz(self, file_name, verbose = 1):
        with open(file_name) as f:
            if(verbose):
                print('Reading the xyz file...')
            
            s_single = Structure()
            for line in f:
                num_atom = int(line.strip('\n'))
                line = f.readline()
                string_lattice = np.array(line.strip('\n').split()[1:])
                string_lattice[-1] = string_lattice[-1][:-1]     # remove the double quotation mark
                string_lattice = string_lattice.astype('float64')
                s_single._lattice = string_lattice.reshape(3, 3)
                positions_single = []
                for i in range(num_atom):
                    line = f.readline()
                    add_atom = line.strip('\n').split()
                    s_single._species.append(add_atom[0])
                    s_single._atom.append([Atom(add_atom[0])])
                    positions_single.append(list(map(float, add_atom[1:])))
                s_single._coords_cartesian = np.array(positions_single)
                s_single.fillna()
                s_single.update_direct()
                s_single_copy = copy.deepcopy(s_single)
                self._structures.append(s_single_copy)
                del s_single
                s_single = Structure()
                if(verbose):
                    print(f'\r{len(self._structures)}', end = '')
        if(verbose):
            print()
    
    def cal_msd(self, index_atom = -1):
        num_atom = len(self._structures[0]._species)
        res_msd = []
        for item in self._structures:
            if(len(item._species) != num_atom):
                print("ERROR: cannot calculate msd with different atom number...")
                break
            if(index_atom >= 0 and index_atom < num_atom):
                x2 = (item._coords_cartesian[index_atom][0] - self._structures[0]._coords_cartesian[index_atom][0])**2
                y2 = (item._coords_cartesian[index_atom][1] - self._structures[0]._coords_cartesian[index_atom][1])**2
                z2 = (item._coords_cartesian[index_atom][2] - self._structures[0]._coords_cartesian[index_atom][2])**2
                res_msd.append(np.sqrt(x2+y2+z2))
            else:
                msd_single = []
                for index, coords_atom in enumerate(item._coords_cartesian):
                    x2 = (coords_atom[0] - self._structures[0]._coords_cartesian[index][0]) ** 2
                    y2 = (coords_atom[1] - self._structures[0]._coords_cartesian[index][1]) ** 2
                    z2 = (coords_atom[2] - self._structures[0]._coords_cartesian[index][2]) ** 2
                    msd_single.append(np.sqrt(x2 + y2 + z2))
                res_msd.append(np.mean(msd_single))
        return np.array(res_msd)
    
    def to_n2p2(self, file_name = "input.data", verbose = 1):
        with open(file_name, 'w') as f:
            if(verbose):
                print('Writing the n2p2 file...')
            
            num_structures = len(self._structures)
            for i in range(num_structures):
                f.write('begin\n')
                f.write('comment Created_by_datana\n')
                f.write(f'lattice {self._structures[i]._lattice[0][0]:.9f} {self._structures[i]._lattice[0][1]:.9f} {self._structures[i]._lattice[0][2]:.9f}\n')
                f.write(f'lattice {self._structures[i]._lattice[1][0]:.9f} {self._structures[i]._lattice[1][1]:.9f} {self._structures[i]._lattice[1][2]:.9f}\n')
                f.write(f'lattice {self._structures[i]._lattice[2][0]:.9f} {self._structures[i]._lattice[2][1]:.9f} {self._structures[i]._lattice[2][2]:.9f}\n')
                for j in range(len(self._structures[i]._species)):
                    f.write(f'atom {self._structures[i]._coords_cartesian[j][0]:.5f} {self._structures[i]._coords_cartesian[j][1]:.5f} {self._structures[i]._coords_cartesian[j][2]:.5f} ')
                    f.write(f'{self._structures[i]._species[j]} 0.0 0.0 ')
                    f.write(f'{self._structures[i]._forces[j][0]:.5f} {self._structures[i]._forces[j][1]:.5f} {self._structures[i]._forces[j][2]:.5f}\n')
                f.write(f'energy {self._structures[i]._energy:.8f}\n')
                f.write('charge 0.00\n')
                f.write('end\n')
                if(verbose):
                    print(f'\r{i+1}/{num_structures}', end = '')
        if(verbose):
            print()
    
    def to_POSCAR(self, verbose = 1, mode = 'Direct'):
        if(verbose):
            print('Writing the POSCAR file...')
        num_structures = len(self._structures)
        for i in range(num_structures):
            self._structures[i].to_POSCAR('POSCAR_'+str(i), mode = mode)
            if(verbose):
                print(f'\r{i+1}/{num_structures}', end = '')
        if(verbose):
            print()
    
    def to_xyz(self, file_name = 'data.xyz', verbose = 1):
        with open(file_name, 'w') as f:
            if(verbose):
                print('Writing the xyz file...')
            num_structures = len(self._structures)
            for i in range(num_structures):
                f.write(f'{len(self._structures[i]._species)}\n')
                f.write(f'Lattice=" {self._structures[i]._lattice[0][0]:.9f} {self._structures[i]._lattice[0][1]:.9f} {self._structures[i]._lattice[0][2]:.9f}')
                f.write(f' {self._structures[i]._lattice[1][0]:.9f} {self._structures[i]._lattice[1][1]:.9f} {self._structures[i]._lattice[1][2]:.9f}')
                f.write(f' {self._structures[i]._lattice[2][0]:.9f} {self._structures[i]._lattice[2][1]:.9f} {self._structures[i]._lattice[2][2]:.9f}"\n')
                for j in range(len(self._structures[i]._species)):
                    f.write(f'{self._structures[i]._species[j]}  {self._structures[i]._coords_cartesian[j][0]:.5f}  {self._structures[i]._coords_cartesian[j][1]:.5f}  {self._structures[i]._coords_cartesian[j][2]:.5f}\n')
                if(verbose):
                    print(f'\r{i+1}/{num_structures}', end = '')
        if(verbose):
            print()



# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.signal import savgol_filter
from scipy.optimize import curve_fit

class Spectrum1D:
    def __init__(self, x_values, y_values):
        '''
        Constructor
        '''
        self._x_values = x_values
        self._y_values = y_values
    
    def __normal_distribution(self, x, mean, sigma):
        '''
        Normal distribution with mean as mean and sigma as sigma
        '''
        return np.exp(-1 * ((x - mean) ** 2) / (2 * (sigma ** 2))) / (np.sqrt(2 * np.pi) * sigma)
    
    def get_values(self):
        '''
        Get x and y values
        '''
        return self._x_values, self._y_values
    
    def set_values(self, x_values, y_values):
        '''
        Set the x and y values manually
        '''
        self._x_values = x_values
        self._y_values = y_values
    
    def smooth(self, method = 'sg', window_length = 9, polyorder = 2):
        y_sg = savgol_filter(self._y_values, window_length, polyorder)
        return Spectrum1D(self._x_values, y_sg)
    
    def find_peak(self, window_length = 5, verbose = 1):
        list_peak = []
        x_min = np.min(self._x_values)
        x_max = np.max(self._x_values)
        x_dx = (x_max - x_min) / len(self._x_values)
        window_step = int(window_length / x_dx)
        for i in range(len(self._y_values)):
            if(self._x_values[i] < x_min + window_length or self._x_values[i] > x_max - window_length):
                continue
            if(self._y_values[i] == np.max(self._y_values[i - window_step : i + window_step])):
                dic_peak = {'x':self._x_values[i], 'height':self._y_values[i]}
                list_peak.append(dic_peak)
        df_peak = pd.DataFrame(list_peak)
        df_peak = df_peak.sort_values('height', ascending = False, ignore_index = True)
        if(verbose):
            if(df_peak.empty):
                print('No peak!')
            else:
                print(f'{len(df_peak)} peak(s) is found')
        return df_peak
    
    def sub_baseline(self, method = 'linear', x_min = 0, x_max = 0):
        if(x_min == 0 and x_max == 0):
            x_min = np.min(self._x_values)
            x_max = np.max(self._x_values)
        dist_min = np.abs(self._x_values - x_min)
        dist_max = np.abs(self._x_values - x_max)
        index_min = np.argsort(dist_min)[0]
        index_max = np.argsort(dist_max)[0]
        slope = (self._y_values[index_max] - self._y_values[index_min]) / (self._x_values[index_max] - self._x_values[index_min])
        intercept = self._y_values[index_max] - slope * self._x_values[index_max]
        bl = slope * self._x_values + intercept
        y_sub = self._y_values - bl
        y_sub -= np.min(y_sub)
        return Spectrum1D(self._x_values, y_sub)
    
    def cal_integral(self, x_min = 0, x_max = 0, sub_bl = True):
        if(x_min == 0 and x_max == 0):
            x_min = self._x_values[0]
            x_max = self._x_values[len(self._x_values) - 1]
        dist_min = np.abs(self._x_values - x_min)
        dist_max = np.abs(self._x_values - x_max)
        index_min = np.argsort(dist_min)[0]
        index_max = np.argsort(dist_max)[0]
        area = 0.0
        for i in range(index_min, index_max):
            area += (self._y_values[i] + self._y_values[i + 1]) * (self._x_values[i + 1] - self._x_values[i]) * 0.5
        if(sub_bl):
            area -= (self._y_values[index_min] + self._y_values[index_max]) * (self._x_values[index_max] - self._x_values[index_min]) * 0.5
        print(f'area value is {area:.4f}')
        return area
        
    def peak_separation(self, num_peak = 1):
        para = np.ones(3 * num_peak)
        spec_peak = self.find_peak(verbose = 0)[:num_peak]
        for item in range(num_peak):
            para[3 * item] = spec_peak.height[item]
            para[3 * item + 1] = spec_peak.x[item]
        def func(x, *arg):
            res = 0
            for i in range(num_peak):
                res += arg[3*i] * self.__normal_distribution(x, arg[3*i+1], arg[3*i+2])
            return res
        pfit, pcov = curve_fit(func, self._x_values, self._y_values, p0 = para, bounds=(0, np.inf))

        res_fit = func(self._x_values, *pfit)
        plt.plot(self._x_values, self._y_values, 'k')
        plt.plot(self._x_values, res_fit, 'r--')
        for i in range(num_peak):
            plt.fill_between(self._x_values, 0, pfit[3*i] * self.__normal_distribution(self._x_values, pfit[3*i+1], pfit[3*i+2]), alpha = 0.5)
        plt.show()
        pfit_peak = [np.round(pfit[i], 4) for i in range(1, len(pfit), 3)]
        print(f'peak: {pfit_peak}')
        return
    
    def plot(self):
        plt.plot(self._x_values, self._y_values, 'k')
        plt.show()
        return


# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
import cv2

class Figure:
    def __init__(self):
        self.flag = 1
    
    def __dfs_nb(self, data_binary, r, c):
        if(r == 0 or r == self.n_row - 1 or c == 0 or c == self.n_col - 1):
            self.flag = 0
            return
        if(data_binary[r][c] == False):
            return
        data_binary[r][c] = False
        self.__dfs_nb(data_binary, r - 1, c)
        self.__dfs_nb(data_binary, r + 1, c)
        self.__dfs_nb(data_binary, r, c - 1)
        self.__dfs_nb(data_binary, r, c + 1)
        return
    
    def __dfs_wb(self, data_binary, r, c):
        if(r < 0 or r > self.n_row - 1 or c < 0 or c > self.n_col - 1):
            return
        if(data_binary[r][c] == False):
            return
        data_binary[r][c] = False
        self.__dfs_wb(data_binary, r - 1, c)
        self.__dfs_wb(data_binary, r + 1, c)
        self.__dfs_wb(data_binary, r, c - 1)
        self.__dfs_wb(data_binary, r, c + 1)
        return
    
    def set_data(self, data):
        self.data = data
        self.n_row = len(self.data)
        self.n_col = len(self.data[0])
        return
    
    def get_data(self):
        return self.data
    
    def imread(self, path, mode = cv2.IMREAD_GRAYSCALE):
        self.data = cv2.imread(path, mode)
        self.n_row = len(self.data)
        self.n_col = len(self.data[0])
    
    def imshow(self):
        plt.imshow(self.data, 'gray')
        plt.xticks([])
        plt.yticks([])
        plt.show()
        
    def blur(self, ksize  = (1, 1)):
        self.data = cv2.blur(self.data, ksize)
        
    def count(self, threshold = 0.5, mode = 'avg'):
        data_binary = self.data > threshold * 255
        plt.imshow(data_binary, 'gray')
        res_nb = 0
        for r in range(self.n_row):
            for c in range(self.n_col):
                if(data_binary[r][c] == True):
                    self.flag = 1
                    self.__dfs_nb(data_binary, r, c)
                    res_nb += self.flag
        data_binary = self.data > threshold * 255
        res_wb = 0
        for r in range(self.n_row):
            for c in range(self.n_col):
                if(data_binary[r][c] == True):
                    self.flag = 1
                    self.__dfs_wb(data_binary, r, c)
                    res_wb += self.flag
        res = 0.5 * (res_nb + res_wb)
        print(f'The number of particles is {res}')
        return res
        
    def flatten(self, num_r = 3, num_c = 3):
        list_r = [int(i * self.n_row / num_r) for i in range(num_r + 1)]
        list_c = [int(i * self.n_col / num_c) for i in range(num_c + 1)]
        
        list_min = []
        for i in range(num_r * num_c):
            data_split = self.data[list_r[i//num_c]:list_r[i//num_c+1], list_c[i%num_c]:list_c[i%num_c+1]]
            index_min = np.argmin(data_split)
            r = int(index_min / len(data_split[0]))
            c = index_min % len(data_split[0])
            list_min.append([r + list_r[i//num_c], c + list_c[i%num_c], data_split[r][c]])
        list_min = np.array(list_min)
        
        def func_plane(coor, a, b, c):
            z = a*coor[0] + b*coor[1] + c
            return z
        coor = [list_min[:,0], list_min[:,1]]
        pfit, pcov = curve_fit(func_plane, coor, list_min[:,2])
        
        plane = []
        for r in range(self.n_row):
            for c in range(self.n_col):
                plane.append(pfit[0] * r + pfit[1] * c + pfit[2])
        plane = np.array(plane)
        plane = plane.reshape(self.n_row, self.n_col)
        
        data_flatten = self.data - plane
        data_min = np.min(data_flatten)
        data_max = np.max(data_flatten)
        minmax = (data_flatten - data_min) / (data_max - data_min)
        if(data_min < 0): data_min = 0
        if(data_max > 255): data_max = 255
        data_flatten = minmax * (data_max - data_min) + data_min
        
        fig_flatten = Figure()
        fig_flatten.set_data(data_flatten)
        
        return fig_flatten
    
    def flatten3p(self, list_coor):  
        x1 = list_coor[0][0]
        y1 = list_coor[0][1]
        z1 = int(self.data[x1][y1])
        x2 = list_coor[1][0]
        y2 = list_coor[1][1]
        z2 = int(self.data[x2][y2])
        x3 = list_coor[2][0]
        y3 = list_coor[2][1]
        z3 = int(self.data[x3][y3])
        
        para_a = (y2 - y1)*(z3 - z1) - (z2 -z1)*(y3 - y1)
        para_b = (x3 - x1)*(z2 - z1) - (x2 - x1)*(z3 - z1)
        para_c = (x2 - x1)*(y3 - y1) - (x3 - x1)*(y2 - y1)
        para_d = -(para_a * x1 + para_b * y1 + para_c * z1)
        
        plane = []
        for r in range(self.n_row):
            for c in range(self.n_col):
                plane.append(-(para_a * r + para_b * c + para_d) / para_c)
        plane = np.array(plane)
        plane = plane.reshape(self.n_row, self.n_col)
        
        data_flatten = self.data - plane
        data_min = np.min(data_flatten)
        data_max = np.max(data_flatten)
        minmax = (data_flatten - data_min) / (data_max - data_min)
        if(data_min < 0): data_min = 0
        if(data_max > 255): data_max = 255
        data_flatten = minmax * (data_max - data_min) + data_min
        
        fig_flatten = Figure()
        fig_flatten.set_data(data_flatten)
        
        return fig_flatten


# -*- coding: utf-8 -*-

import numpy as np
import copy
from structure import Structure

class Dynamic:
    def __init__(self):
        self._temperature = []
        self._energy = []
        self._structures = []
        
    def read_OSZICAR_XDATCAR(self, OSZICAR = "OSZICAR", XDATCAR = "XDATCAR"):
        list_temperature = []
        list_energy = []
        list_structure = []
        if(OSZICAR):
            with open(OSZICAR) as f:
                for line in f:
                    if("T=" in line):
                        list_line = line.strip().split()
                        list_temperature.append(float(list_line[2]))
                        list_energy.append(float(list_line[4]))
        else:
            pass
        if(XDATCAR):
            with open(XDATCAR) as f:
                s_single = Structure()
                temp_lattice = []
                for i in range(2):
                    f.readline()
                for i in range(3):
                    data = f.readline().strip().split()
                    data = list(map(float, data))
                    temp_lattice.append(data)
                s_single._lattice = np.array(temp_lattice)
                list_species = f.readline().strip().split()
                num_atom = f.readline().strip().split()
                num_atom = list(map(int, num_atom))
                for index, value in enumerate(list_species):
                    s_single._species += [value] * num_atom[index]
                for line in f:
                    temp_coords_direct = []
                    for i in range(int(np.sum(num_atom))):
                        data = f.readline().split()
                        temp_coords_direct.append(data)
                    s_single._coords_direct = np.array(temp_coords_direct).astype('float64')
                    s_single.update_cartesian()
                    s_single_copy = copy.deepcopy(s_single)
                    list_structure.append(s_single_copy)
        else:
            pass
        if(not len(list_temperature)):
            list_temperature, list_energy = [0] * len(list_structure), [0] * len(list_structure)
        elif(not len(list_structure)):
            s_single = Structure()
            list_structure = [s_single] * len(list_temperature)
        if(len(list_energy) != len(list_structure)):
            print("ERROR: Energy and structure list must the same size...")
            return        
        self._temperature = np.array(list_temperature)
        self._energy = np.array(list_energy)
        self._structures = list_structure
    
    def update_OSZICAR(self, OSZICAR = "OSZICAR"):
        list_temperature = []
        list_energy = []
        with open(OSZICAR) as f:
            for line in f:
                if("T=" in line):
                    list_line = line.strip().split()
                    list_temperature.append(float(list_line[2]))
                    list_energy.append(float(list_line[4]))
        if(len(list_energy) == len(self._structures)):
            self._temperature = np.array(list_temperature)
            self._energy = np.array(list_energy)
    
    def update_XDATCAR(self, XDATCAR = "XDATCAR"):
        list_structure = []
        with open(XDATCAR) as f:
            s_single = Structure()
            temp_lattice = []
            for i in range(2):
                f.readline()
            for i in range(3):
                data = f.readline().strip().split()
                data = list(map(float, data))
                temp_lattice.append(data)
            s_single._lattice = np.array(temp_lattice)
            list_species = f.readline().strip().split()
            num_atom = f.readline().strip().split()
            num_atom = list(map(int, num_atom))
            for index, value in enumerate(list_species):
                s_single._species += [value] * num_atom[index]
            for line in f:
                temp_coords_direct = []
                for i in range(int(np.sum(num_atom))):
                    data = f.readline().split()
                    temp_coords_direct.append(data)
                s_single._coords_direct = np.array(temp_coords_direct).astype('float64')
                s_single.update_cartesian()
                s_single_copy = copy.deepcopy(s_single)
                list_structure.append(s_single_copy)
        if(len(self._structures) == len(list_energy)):
            self._structures = list_structure
    
    def get_temperature(self):
        return self._temperature
        
    def get_energy(self):
        return self._energy
    
    def cal_msd(self, index_atom = -1):
        num_atom = len(self._structures[0]._species)
        res_msd = []
        for item in self._structures:
            if(len(item._species) != num_atom):
                print("ERROR: cannot calculate msd with different atom number...")
                break
            if(index_atom >= 0 and index_atom < num_atom):
                x2 = (item._coords_cartesian[index_atom][0] - self._structures[0]._coords_cartesian[index_atom][0])**2
                y2 = (item._coords_cartesian[index_atom][1] - self._structures[0]._coords_cartesian[index_atom][1])**2
                z2 = (item._coords_cartesian[index_atom][2] - self._structures[0]._coords_cartesian[index_atom][2])**2
                res_msd.append(np.sqrt(x2+y2+z2))
            else:
                msd_single = []
                for index, coords_atom in enumerate(item._coords_cartesian):
                    x2 = (coords_atom[0] - self._structures[0]._coords_cartesian[index][0]) ** 2
                    y2 = (coords_atom[1] - self._structures[0]._coords_cartesian[index][1]) ** 2
                    z2 = (coords_atom[2] - self._structures[0]._coords_cartesian[index][2]) ** 2
                    msd_single.append(np.sqrt(x2 + y2 + z2))
                res_msd.append(np.mean(msd_single))
        return np.array(res_msd)


import setuptools

setuptools.setup(
	name = 'datana',
	version = '0.3.1.0',
	author = 'MasterLegend',
	description = 'datana pakage',
	long_description = 'Datana is a package for data processing in the field of chemistry. It can be used to process both computational and experimental data.',
	long_description_content_type = 'text/markdown',
	url = 'https://github.com/MasterLegend/datana',
	py_modules = ['datana.spectrum', 'datana.figure', 'datana.atom', 'datana.structure', 'datana.dynamic', 'datana.dos', 'datana.plotting'],
)


# -*- coding: utf-8 -*-

import pyvista as pv
from pyvistaqt import BackgroundPlotter
from structure import Structure

def plot_structure(s):
    
    plotter = BackgroundPlotter()
    
    for i in range(len(s._coords_cartesian)):
         _ = plotter.add_mesh(pv.Sphere(radius=s.get_atom(i).get_radius(), center = s._coords_cartesian[i]))
    
    _ = plotter.add_mesh(pv.Line(pointa=(0.0, 0.0, 0.0), pointb=s._lattice[0], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=(0.0, 0.0, 0.0), pointb=s._lattice[1], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[0], pointb=s._lattice[0]+s._lattice[1], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[1], pointb=s._lattice[0]+s._lattice[1], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=(0.0, 0.0, 0.0), pointb=s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[0], pointb=s._lattice[0]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[1], pointb=s._lattice[1]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[0]+s._lattice[1], pointb=s._lattice[0]+s._lattice[1]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[2], pointb=s._lattice[0]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[2], pointb=s._lattice[1]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[0]+s._lattice[2], pointb=s._lattice[0]+s._lattice[1]+s._lattice[2], resolution=1), color='white')
    _ = plotter.add_mesh(pv.Line(pointa=s._lattice[1]+s._lattice[2], pointb=s._lattice[0]+s._lattice[1]+s._lattice[2], resolution=1), color='white')
    
    return